<?php

namespace SM\Service;

use SM\Factory\Sm00049AgrupamentoDeContratosFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use SM\Service\ProcedureService;


class Sm00049AgrupamentoDeContratosService
{

    protected $AgrupamentoFactory;
    protected $Environment;
    protected $newCode;
    protected $procedureService;

    public function __construct(EntityManager $entityManager, Sm00049AgrupamentoDeContratosFactory $AgrupamentoFactory, Environment $Environment, NewCode $newCode, ProcedureService $procedureService)
    {
        $this->AgrupamentoFactory = $AgrupamentoFactory;
        $this->connection         = $entityManager->getConnection();
        $this->Environment        = $Environment;
        $this->newCode            = $newCode;
        $this->procedureService   = $procedureService;
    }


    public function getInformacoesGridPrincipal()
    {
        return $this->AgrupamentoFactory->getInformacoesGridPrincipal();
    }

    public function getParceirosDeNegocio($parametros)
    {
        $cdOperador = $this->Environment->getCdOperador();

        $tiposParceiro = array();
        foreach ($parametros['CDTIPOPARCEIRO'] as $tipo) {
            $tiposParceiro[] = array_shift($tipo);
        }

        $tiposParceiro = implode("','", $tiposParceiro);
        $tiposParceiro = "'$tiposParceiro'";

        $param = array(
            "CDOPERADOR" => $cdOperador,
            "CDTIPOPARCEIRO" => $tiposParceiro
        );

        return $this->AgrupamentoFactory->getParceirosDeNegocio($param);
    }

    public function getEscoposFaturamento()
    {
        return $this->AgrupamentoFactory->getEscoposFaturamento();
    }

    public function getTipoOperacao()
    {
        return $this->AgrupamentoFactory->getTipoOperacao();
    }

    public function getSeriesNFAgrupamento()
    {
        return $this->AgrupamentoFactory->getSeriesNFAgrupamento();
    }

    public function getFilialAgrupamento()
    {
        return $this->AgrupamentoFactory->getFilialAgrupamento();
    }

    public function getContratosAgrupamento($parametros)
    {
        $escopos = array();
        foreach ($parametros["NRTIPOORCAMENTO"] as $escopo) {
            $escopos[] = array_shift($escopo);
        }

        $escopos = implode(",", $escopos);

        $param = array(
            "NRPARCFATURA"    => $parametros["NRPARCFATURA"],
            "NRTIPOORCAMENTO" => $escopos,
            "NRAGRUPAMENTO" => $parametros["NRAGRUPAMENTO"],
        );

        return $this->AgrupamentoFactory->getContratosAgrupamento($param);
    }

    public function getInformacoesGridContratos($parametros)
    {
        $nrorg = $this->Environment->getNrorg();

        $param = array(
            "NRAGRUPAMENTO" => $parametros["NRAGRUPAMENTO"],
            "NRORG"         => $nrorg
        );

        return $this->AgrupamentoFactory->getInformacoesGridContratos($param);
    }

    public function validarEscoposAgrupamento($parametros)
    {
        $nrorg      = $this->Environment->getNrorg();
        $strEscopos = implode(",", $parametros["NRTIPOORCAMENTO"]);

        $paramsFaturaEscopo = $this->AgrupamentoFactory->getParametrosFaturamentoEscopo($strEscopos, $nrorg);

        if (count($paramsFaturaEscopo) > 1) {
            throw new \Exception("PARAMS_DIFERENTE");
        } else {
            $competencias = $this->AgrupamentoFactory->getCompetenciasFaturamentoEscopo($strEscopos, $nrorg);
            foreach ($competencias as $competencia) {
                foreach ($parametros["NRTIPOORCAMENTO"] as $escopo) {
                    if (strstr($competencia["NRTIPOORCAMENTO"], '_' . $escopo . '_') == false) {
                        throw new \Exception("COMPET_DIFERENTE");
                    }
                }
            }

            $paramsDoServico = $this->AgrupamentoFactory->getParametrosFaturamentoServicoDoEscopo($strEscopos, $nrorg);
            if (count($paramsDoServico) > 1) {
                return false;
            } else {
                $competenciasServico = $this->AgrupamentoFactory->getCompetenciasFaturamentoServicoDoEscopo($strEscopos, $nrorg);
                foreach ($competenciasServico as $competencia) {
                    foreach ($parametros['NRTIPOORCAMENTO'] as $escopo) {
                        if (strstr($competencia["NRTIPOORCAMENTO"], '_' . $escopo . '_') == false) {
                            return false;
                        }
                    }
                }
                return true;
            }
        }
    }

    public function salvarAgrupamento($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();
            $strEscopos = implode(",", $parametros['NRTIPOORCAMENTO']);

            $escopos = $this->AgrupamentoFactory->getParametrosFaturamentoEscopo($strEscopos, $nrorg);

            $nragrupamento = $this->newCode->getCodeByTableName("FCT_AGRUPAMENTO", $nrorg, 1, 10, true);

            $params = array(
                "CDTIPOOPERFATURA" => $escopos[0]['CDTIPOOPERFATURA'],
                "CDSERIENF"        => $escopos[0]['CDSERIENF'],
                "CDEMPRESA"        => $escopos[0]['CDEMPRESA'],
                "CDINSCESTA"       => $escopos[0]['CDINSCESTA'],
                "CDINSCMUNI"       => $escopos[0]['CDINSCMUNI'],
                "CDFILIAL"         => $escopos[0]['CDFILIALFATURA'],
                "NRAGRUPAMENTO"    => $nragrupamento,
                "NMAGRUPAMENTO"    => $parametros['NMAGRUPAMENTO'],
                "DSOBSAGRUPAMENTO" => $parametros['DSOBSAGRUPAMENTO'],
                "NRPARCFATURA"     => $parametros['NRPARCNEGOCIO'],
                "NRORG"            => $nrorg,
                "IDATIVO"          => 'S',
                "NRORGINCLUSAO"    => $nrorg,
                "CDOPERINCLUSAO"   => $cdoperador
            );

            $this->AgrupamentoFactory->salvarAgrupamento($params);

            foreach ($parametros['NRTIPOORCAMENTO'] as $escopo) {
                $nrescopoagrup = $this->newCode->getCodeByTableName("FCT_RELACESCOPOAGRUPAMENTO", $nrorg, 1, 10, true);

                $paramsEsc = array(
                    "NRRELACESCAGRUP" => $nrescopoagrup,
                    "NRTIPOORCAMENTO" => $escopo,
                    "NRAGRUPAMENTO"   => $nragrupamento,
                    "NRORG"           => $nrorg,
                    "NRORGINCLUSAO"   => $nrorg,
                    "CDOPERINCLUSAO"  => $cdoperador,
                    "IDATIVO"         => "S"
                );

                $this->AgrupamentoFactory->salvarEscoposAgrupamento($paramsEsc);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function updateAgrupamento($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();

            $params = array(
                "NMAGRUPAMENTO"    => $parametros['NMAGRUPAMENTO'],
                "DSOBSAGRUPAMENTO" => $parametros['DSOBSAGRUPAMENTO'],
                "NRORGULTATU"      => $nrorg,
                "CDOPERULTATU"     => $cdoperador,
                "NRAGRUPAMENTO"    => $parametros['NRAGRUPAMENTO']
            );

            $this->AgrupamentoFactory->updateAgrupamento($params);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deletarAgrupamento($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();

            $params = array(
                "NRAGRUPAMENTO" => $parametros['NRAGRUPAMENTO'],
                "NRORG"         => $nrorg
            );
            $nrseqcontratos = $this->AgrupamentoFactory->getQtdContratosAgrupamento($params);

            if (count($nrseqcontratos) > 0) {
                $paramsExclusao = array(
                    "NRAGRUPAMENTO" => $parametros['NRAGRUPAMENTO'],
                    "NRSEQCONTRATO" => implode(",", $nrseqcontratos)
                );

                $this->deletarContratos($paramsExclusao);
            }

            $this->AgrupamentoFactory->deletarEscoposAgrupamento($params);
            $this->AgrupamentoFactory->deletarAgrupamento($params);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deletarContratos($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();

            $params = array(
                "NRAGRUPAMENTO" => $parametros['NRAGRUPAMENTO'],
                "NRORG"         => $nrorg,
                "NRSEQCONTRATO" => $parametros['NRSEQCONTRATO']
            );

            $nrfaturaconts = $this->AgrupamentoFactory->getParcelasAgrupamentoCont($params);

            if (count($nrfaturaconts) > 0) {
                foreach ($nrfaturaconts as $nrfaturacont) {
                    $parametrosDesagrupar = array(
                        "NRFATURACONT"  => $nrfaturacont['NRFATURACONT'],
                        "NRSEQCONTRATO" => $parametros['NRSEQCONTRATO'],
                        "IDACAO"        => "DESAGRUPA",
                        "NRORG"         => $nrorg,
                        "CDOPERADOR"    => $cdoperador
                    );

                    $this->desagruparParcelasFaturamento($parametrosDesagrupar);
                }
            }

            $this->AgrupamentoFactory->deletarContratoAgrupamento($params);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function desagruparParcelasFaturamento($parametros)
    {
        $this->connection->beginTransaction();

        // parametros recebidos da função "deletarContrato"
        $nrfaturacont = $parametros['NRFATURACONT'];
        $nrseqcontratos = $parametros['NRSEQCONTRATO'];
        $idAcao = $parametros['IDACAO'];
        $nrorg = $parametros['NRORG'];
        $operador = $this->Environment->getCdOperador();

        try {
            $params = array(
                "NRFATURACONT" => $nrfaturacont,
                "NRORG"        => $nrorg
            );

            $dadosParcela = $this->AgrupamentoFactory->getDadosParcela($params);

            if (!isset($dadosParcela)) {
                throw new \Exception("Operação bloqueada!<br>Falha ao efetuar o desmembramento de contratos.");
            } else {
                $vrvencimentoold = 0;
                $dadosParcela = $dadosParcela[0];

                foreach (explode(',', $nrseqcontratos) as $nrseqcontrato) {
                    $vrTotalParcela = 0;

                    $paramsItems = array(
                        "NRORG"         => $nrorg,
                        "NRSEQCONTRATO" => $nrseqcontrato,
                        "NRFATURACONT"  => $nrfaturacont
                    );

                    $dadosItem = $this->AgrupamentoFactory->getItemsFatura($paramsItems);

                    foreach ($dadosItem as $dadoItem) {
                        $vrTotalParcela = $vrTotalParcela + $dadoItem["VRVENCIMENTO"];
                    }

                    $dadosParcela["VRVENCIMENTO"] = $vrTotalParcela;
                    $vrvencimentoold = $vrvencimentoold + $vrTotalParcela;

                    $newnrfaturacont = $this->inserirParcela($dadosParcela, $nrseqcontrato, $nrorg, $operador, null);

                    $this->alterarRelacContratosFatura($nrfaturacont, $nrseqcontrato, $idAcao, $nrorg, $operador, $newnrfaturacont);

                    foreach ($dadosItem as $dadoItem) {
                        $this->inserirItensFatura($dadoItem, $nrseqcontrato, $newnrfaturacont, null, $nrorg, $operador);
                    }
                }

                $this->validarQuantidadeContratosParcela($nrfaturacont, $nrorg, $operador, $vrvencimentoold, $nrseqcontratos);
                $this->connection->commit();
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inserirParcela($dadosParcela, $contrato, $nrorg, $operador, $dsJustificativa)
    {
        $newnrfaturacont = $this->newCode->getCodeByTableName("FCT_FATURACONT", $nrorg, 1, 10, true);

        $nrparcela = null;
        if (!empty($contrato)) {
            $params = array(
                "NRSEQCONTRATO" => $contrato,
                "NRORG"         => $nrorg
            );
            $dataset = $this->AgrupamentoFactory->getQuantidadeParcelasContrato($params);
            $nrparcela = (int)$dataset[0]["CONTADOR"] + 1;
        }

        $params = array(
            "NRFATURACONT"       => $newnrfaturacont,
            "NRSEQCONTRATO"      => null, //$contrato,
            "NRGRUPOFATURA"      => $dadosParcela["NRGRUPOFATURA"],
            "CDFILIALFATURA"     => $dadosParcela["CDFILIALFATURA"],
            "NRPARCFATURA"       => $dadosParcela["NRPARCFATURA"],
            "IDTPFATURA"         => $dadosParcela["IDTPFATURA"],
            "IDSITUACAO"         => $dadosParcela["IDSITUACAO"],
            "DTINICOMPET"        => $dadosParcela["DTINICOMPET"],
            "DTFINCOMPET"        => $dadosParcela["DTFINCOMPET"],
            "DTLIMITEAPU"        => $dadosParcela["DTLIMITEAPU"],
            "DTVENCIMENTO"       => $dadosParcela["DTVENCIMENTO"],
            "NRDIASVENCIMENTO"   => $dadosParcela["NRDIASVENCIMENTO"],
            "QTDIASFATUVENC"     => $dadosParcela["QTDIASFATUVENC"],
            "VRVENCIMENTO"       => $dadosParcela["VRVENCIMENTO"],
            "VRFALTAFATURAR"     => $dadosParcela["VRVENCIMENTO"],
            "IDFORMAFATURA"      => $dadosParcela["IDFORMAFATURA"],
            "DSJUSTIFICATIVA"    => $dsJustificativa,
            "NRAPURACAO"         => $dadosParcela["NRAPURACAO"],
            "NRMEDICAOFATURA"    => $dadosParcela["NRMEDICAOFATURA"],
            "CDTIPOOPERFATURA"   => $dadosParcela["CDTIPOOPERFATURA"],
            "CDSERIENF"          => $dadosParcela["CDSERIENF"],
            "CDEMPRESA"          => $dadosParcela["CDEMPRESA"],
            "CDINSCESTA"         => $dadosParcela["CDINSCESTA"],
            "CDINSCMUNI"         => $dadosParcela["CDINSCMUNI"],
            "VRFATURADO"         => 0,
            "NRORG"              => $nrorg,
            "OPERADOR"           => $operador,
            "NRPARCELA"          => $nrparcela
        );

        $this->AgrupamentoFactory->insereParcelaDesmembramento($params);

        return $newnrfaturacont;
    }

    /* Insere na tabela FCT_RELACAGRUPCONTFAT os contratos e medições ligados as
       parcelas agrupadas */
    public function alterarRelacContratosFatura($nrfaturaconts, $nrseqcontrato, $idAcao, $nrorg, $operador, $newnrfaturacont)
    {
        try {
            $paramsRelac = array(
                "NRSEQCONTRATO" => $nrseqcontrato,
                "NRORG"         => $nrorg
            );

            $relacParcelas = $this->AgrupamentoFactory->getInformacoesRelacParcelaCont($nrfaturaconts, $paramsRelac);

            foreach ($relacParcelas as $relacParcela) {
                $this->inserirRelacAgrupContFat($relacParcela["NRSEQCONTRATO"], $newnrfaturacont, $nrorg, $operador, $idAcao, $relacParcela["NRMEDICAOFATURA"]);
            }

            $this->AgrupamentoFactory->deletarRelacAgrupamentoContFat($nrfaturaconts, $paramsRelac);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function inserirRelacAgrupContFat($nrseqcontrato, $nrfaturacont, $nrorg, $operador, $idAcao, $nrmedicaofatura)
    {

        $nragrupcontfat = $this->newCode->getCodeByTableName("FCT_RELACAGRUPCONTFAT", $nrorg, 1, 10, true);

        $params = array(
            "NRAGRUPCONTFAT"  => $nragrupcontfat,
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRORG"           => $nrorg,
            "OPERADOR"        => $operador,
            "NRFATURACONT"    => $nrfaturacont,
            "IDPARCAGRUP"     => ($idAcao == "AGRUPA") ? "S" : "N",
            "NRMEDICAOFATURA" => $nrmedicaofatura
        );
        $this->AgrupamentoFactory->salvarRelacAgrupContFat($params);
    }

    // Insere os itens das respectivas novas parcelas criadas
    public function inserirItensFatura($dadosItem, $contrato, $newnrfaturacont, $dsJustificativa, $nrorg, $operador)
    {

        $newnrfaturaitem = $this->newCode->getCodeByTableName("FCT_ITFATURACONT", $nrorg, 1, 10, true);

        $params = array(
            "NRITFATURACONT"     => $newnrfaturaitem,
            "NRFATURACONT"       => $newnrfaturacont,
            "NRSEQCONTRATO"      => $contrato,
            "DSJUSTIFICATIVA"    => $dsJustificativa,
            "VRFATURADO"         => 0,
            "NRITEMOCONTRATO"    => $dadosItem["NRITEMOCONTRATO"],
            "CDPRODFATURA"       => $dadosItem["CDPRODFATURA"],
            "IDSITUACAO"         => $dadosItem["IDSITUACAO"],
            "VRVENCIMENTO"       => $dadosItem["VRVENCIMENTO"],
            "VRFALTAFATURAR"     => $dadosItem["VRVENCIMENTO"],
            "IDFORMAFATURA"      => $dadosItem["IDFORMAFATURA"],
            "NRAPURACAO"         => $dadosItem["NRAPURACAO"],
            "NRORG"              => $nrorg,
            "OPERADOR"           => $operador
        );

        $this->AgrupamentoFactory->insereItemDesmembramento($params);
    }

    // Verifica se a parcela ainda é um agrupamento
    public function validarQuantidadeContratosParcela($nrfaturacont, $nrorg, $operador, $vrvencimentoold, $nrseqcontratos)
    {
        $params = array(
            "NRSEQCONTRATO" => null,
            "NRORG"         => $nrorg
        );

        $relacParcelas = $this->AgrupamentoFactory->getInformacoesRelacParcelaCont($nrfaturacont, $params);

        if (count($relacParcelas) == 0) {
            $this->AgrupamentoFactory->deletarParcelaFatura($nrfaturacont, $nrorg);
        } else {
            if (count($relacParcelas) == 1) {
                $params = array(
                    "NRFATURACONT" => $nrfaturacont,
                    "NRORG"        => $nrorg,
                    "CDOPERADOR"   => $operador
                );
                $this->AgrupamentoFactory->alterarInfoAgrupamentoParcela($params);
            }

            $params = array(
                "NRFATURACONT"    => $nrfaturacont,
                "NRORG"           => $nrorg,
                "VRVENCIMENTO"    => $vrvencimentoold,
                "DSJUSTIFICATIVA" => null
            );
            $this->AgrupamentoFactory->atualizaValorParcelaOrigem($params);
        }

        foreach (explode(',', $nrseqcontratos) as $nrseqcontrato) {
            $params = array(
                "NRITFATURACONT"   => null,
                "NRFATURACONT"     => $nrfaturacont,
                "NRSEQCONTRATO"    => $nrseqcontrato,
                "NRORG"            => $nrorg
            );

            $this->AgrupamentoFactory->deletaItemParcelaOrigem($params);
        }
    }

    public function validarContratosAprovados($parametros)
    {

        $param = array(
            "NRSEQCONTRATO" => implode(',', $parametros['NRSEQCONTRATO'])
        );

        return $this->AgrupamentoFactory->validarContratosAprovados($param);
    }

    public function salvarContratos($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();

            $params = array(
                "NRSEQCONTRATO" => $parametros['NRSEQCONTRATO'],
                "NRAGRUPAMENTO" => $parametros['NRAGRUPAMENTO'],
                "NRORG"         => $nrorg
            );

            $this->validarContratosAgrupamento($params);

            foreach ($parametros['NRSEQCONTRATO'] as $contrato) {
                $nragrupamentocont = $this->newCode->getCodeByTableName("FCT_RELACAGRUPAMENTOCONT", $nrorg, 1, 10, true);

                $params = array(
                    "NRAGRUPAMENTOCONT" => $nragrupamentocont,
                    "NRAGRUPAMENTO"     => $parametros['NRAGRUPAMENTO'],
                    "NRSEQCONTRATO"     => $contrato,
                    "NRORG"             => $nrorg,
                    "OPERADOR"          => $cdoperador
                );

                /* Insere o Contrato no Agrupamento */
                $this->AgrupamentoFactory->insereContrAgrup($params);

                /* Se o contrato estiver aprovado, regera as parcelas */
                if ($parametros['IDAPROVADOS'] == 'S') {
                    $this->procedureService->executarFctGerarParcelasFatura($contrato, $nrorg, $cdoperador);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function validarContratosAgrupamento($parametros)
    {
        $nrseqcontratos = implode(",", $parametros['NRSEQCONTRATO']);

        $params = array(
            "NRAGRUPAMENTO" => $parametros['NRAGRUPAMENTO'],
            "NRORG"         => $parametros['NRORG']
        );

        $paramContratos = $this->AgrupamentoFactory->getParametrosFatContratos($nrseqcontratos, $params);

        if (count($paramContratos) > 1) {
            throw new \Exception("PARC_DIFERENTES");
        }
    }
}
